"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Mission = void 0;
const uuid_1 = require("uuid");
const config_1 = require("../../common/config");
const default_1 = require("../../common/default");
const types_1 = require("../../common/types");
const error_1 = require("../../error");
const location_1 = require("../location");
class Mission extends default_1.Default {
    constructor(type, id) {
        error_1.MissionError.checkLength(id, config_1.MAX_ID_LENGTH);
        super();
        this.type = type;
        this.id = id || uuid_1.v4();
    }
    setFreeText(freeText) {
        error_1.MissionError.checkLength(freeText, config_1.MAX_FREETEXT_LENGTH);
        this.freeText = freeText;
        return this;
    }
    setMainMissionId(mainId) {
        error_1.MissionError.checkLength(mainId, config_1.MAX_ID_LENGTH);
        this.mainMissionId = mainId;
        return this;
    }
    setOrgId(orgId) {
        error_1.MissionError.checkLength(orgId, config_1.MAX_ID_LENGTH);
        this.orgId = orgId;
        return this;
    }
    setName(name) {
        error_1.MissionError.checkLength(name, config_1.MAX_NAME_X2_LENGTH);
        this.name = name;
        return this;
    }
    setStatus(status, completeness) {
        if (status === types_1.MissionStatus.IN_PROGRESS && completeness)
            if (completeness > 0 && completeness < 100)
                this.status = types_1.MissionStatus.IN_PROGRESS + completeness.toString().padStart(2, '0');
            else
                throw new error_1.MissionError('Percentage of completeness of the mission should be between 1 and 99');
        else
            this.status = status;
        return this;
    }
    setStartTime(start) {
        this.startTime = start.toISOString();
        return this;
    }
    setEndTime(end) {
        this.endTime = end.toISOString();
        return this;
    }
    addResourceIds(resourceIds) {
        resourceIds.forEach(add => error_1.MissionError.checkLength(add, config_1.MAX_RESOURCE_ID_LENGTH));
        if (!this.resourceId)
            this.resourceId = new Array();
        this.resourceId.push(...resourceIds);
        return this;
    }
    addParentMissionIds(parentIds) {
        parentIds.forEach(add => error_1.MissionError.checkLength(add, config_1.MAX_ID_LENGTH));
        if (!this.parentMissionId)
            this.parentMissionId = new Array();
        this.parentMissionId.push(...parentIds);
        return this;
    }
    addChildMissionIds(childIds) {
        childIds.forEach(add => error_1.MissionError.checkLength(add, config_1.MAX_ID_LENGTH));
        if (!this.childMissionId)
            this.childMissionId = new Array();
        this.childMissionId.push(...childIds);
        return this;
    }
    setPosition(position) {
        this.position = position;
        return this;
    }
    setPriority(priority) {
        this.priority = priority;
        return this;
    }
    static default() {
        return new Mission('');
    }
    assign(source) {
        let key;
        const keys = Object.keys(source);
        if ((key = keys.find(f => f === 'type')))
            this.type = source[key];
        if ((key = keys.find(f => f === 'freeText')))
            this.freeText = source[key];
        if ((key = keys.find(f => f === 'id')))
            this.id = source[key];
        if ((key = keys.find(f => f === 'mainMissionId')))
            this.mainMissionId = source[key];
        if ((key = keys.find(f => f === 'orgId')))
            this.orgId = source[key];
        if ((key = keys.find(f => f === 'name')))
            this.name = source[key];
        if ((key = keys.find(f => f === 'status')))
            this.status = source[key];
        if ((key = keys.find(f => f === 'startTime')))
            this.startTime = source[key];
        if ((key = keys.find(f => f === 'endTime')))
            this.endTime = source[key];
        if ((key = keys.find(f => f === 'resourceId'))) {
            this.resourceId = new Array();
            if (source[key] instanceof Array)
                source[key].forEach((add) => { var _a; return (_a = this.resourceId) === null || _a === void 0 ? void 0 : _a.push(add); });
            else
                this.resourceId.push(source[key]);
        }
        if ((key = keys.find(f => f === 'parentMissionId'))) {
            this.parentMissionId = new Array();
            if (source[key] instanceof Array)
                source[key].forEach((add) => { var _a; return (_a = this.parentMissionId) === null || _a === void 0 ? void 0 : _a.push(add); });
            else
                this.parentMissionId.push(source[key]);
        }
        if ((key = keys.find(f => f === 'childMissionId'))) {
            this.childMissionId = new Array();
            if (source[key] instanceof Array)
                source[key].forEach((add) => { var _a; return (_a = this.childMissionId) === null || _a === void 0 ? void 0 : _a.push(add); });
            else
                this.childMissionId.push(source[key]);
        }
        if ((key = keys.find(f => f === 'position')))
            this.position = location_1.Position.default().assign(source[key]);
        if ((key = keys.find(f => f === 'priority')))
            this.priority = source[key];
        return this;
    }
}
exports.Mission = Mission;
